/*
 * A1Process.java
 *
 * Created on 9 July 2002, 10:21
 */

import javax.swing.*;

/**
 * A process is associated with a text area and is scheduled by your dispatcher.
 * Rather than having a PCB and a process table, all necessary process information is kept here.
 *
 * @author  Robert Sheehan
 */
public abstract class A1Process extends Thread {
    
    /* The process states. */
    public static final int BEING_CREATED = 0;
    public static final int RUNNING = 1;
    public static final int READY = 2;
    public static final int WAITING = 3;
    public static final int FINISHED = 4;
    
    /** The output area for this process. */
    private ProcessArea outputArea;
    /** The value of the next identifier to be allocated. */
    private static int nextID = 0;
    /** This process ID. */
    private final int procID = nextID++;
    /** Whether the process should pause now. */
    protected volatile boolean pause = true;
    /** Whether the process should still be alive. */
    protected volatile boolean live = true;
    /** The current state of the process. */
    protected int state;
    /** The number of loops this process will perform.
     * -1 means it will loop forever. */
    protected int loopCount;
    
    /** Creates a new instance of A1Process.
     * @param area the corresponding ProcessArea
     */
    public A1Process(ProcessArea area) {
        state = BEING_CREATED;
        outputArea = area;
        IOSystem.print(this, toString());
        start();
    }
    
    /** The String version of this process. */
    public String toString() {
        return "Process "+procID;
    }
    
    /** Terminates a process. */
    public void terminateProcess() {
        live = false;
        interrupt();
    }
    
    /** Interrupts a process.
     * Should only be called on the currently RUNNING process.
     */
    public void interruptProcess() {
        pause = true;
        interrupt();
    }
    
    /** Dispatches a process. */
    public synchronized void dispatchProcess() {
        state = RUNNING;
        System.out.println(this + " running.");
        pause = false;
        notify();
    }
    
    /** Makes a process wait.
     * The only thing it waits for is a String of keyboard input.
     */
    public synchronized void waitProcess() {
        state = WAITING;
        System.out.println(this + " waiting.");
        while (state == WAITING) {
            try {
                wait();
            }
            catch (InterruptedException ex) {
                System.out.println(this + " interrupted while waiting");
            }
        }
    }
        
    /** Checks to see if the process should block.
     * Blocks the process if necessary.
     * This is not how it would work on a real OS.
     */
    protected synchronized void checkForInterrupts() {
        if (interrupted())
            System.out.println(this + " interrupted.2");
        if (pause) {
            state = READY; // the state only changes when it is really being paused
            // This means the dispatcher can check the state to determine when it really has paused.
            System.out.println(this + " ready.");
            while (live && pause) {
                try {
                    wait();
                }
                catch (InterruptedException ex) {
                    System.out.println(this+" interrupted while paused.");
                }
            }
        }
    }
    
    /**
     * The basic processing routine all processes perform.
     * Actually just a fake. Puts the thread to sleep for a second.
     * If another process should run this can be interrupted.
     */
    protected void doSomeWork() {
        IOSystem.print(this, "*");
        try {
            sleep(1000);
        }
        catch (InterruptedException ex) {
            System.out.println(this + " interrupted.");
        }
    }
    
    /**
     * @return The state of the process.
     */
    public int getState() {
        return state;
    }
    
    /**
     * Sets the process state.
     * @param newState the new state
     */
    public void setState(int newState) {
        state = newState;
    }
    
    /**
     * @return the text area associated with this process.
     */
    public ProcessArea getOutputArea() {
        return outputArea;
    }
    
    public void run() {
        execute();
        state = FINISHED;
        System.out.println(this + " finished.");
        IOSystem.exit(this);
    }
    
    /**
     * The real work of the process.
     */
    protected abstract void execute();
    
}
